unit Saucer;

(*********************************************
TSaucer-TDIBSprite

The saucer class is specific to this app (see
the USES clause under implementation).
*********************************************)
interface

uses

  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, WinGPnt, Sprite,
  LkupTabl, DIB, DIBSprit, Utility, Missile, Ship;

type

  TSaucerType = ( stAthena, stAnders, stUs );

  TSaucer = class( TDIBSprite )
  private
     st: TSaucerType;
     FFireDelay: byte;
     FFire: byte;
  protected
     function GetScore: integer;
  public
     constructor CreateSaucer( nWave: integer );
     procedure Move; override;
     property Score: integer read GetScore;
  end;

implementation

uses

  Main; {specific to this app!}

constructor TSaucer.CreateSaucer( nWave: integer );
begin
  if nWave > 5 then
     begin
        if Random( 20 ) > nWave then
           st := stUs
        else if Random( 10 ) > nWave then
           st := stAnders
        else
           st := stUs;
     end
  else
     begin
        if Random( 10 ) > nWave then
           st := stAthena
        else
           st := stUs;
     end;
  case st of
     stAthena:
        begin
           inherited CreateDIBSprite( frmMain.dibAthena );
           FFireDelay := 80;
        end;
     stAnders:
        begin
           inherited CreateDIBSprite( frmMain.dibAnders );
           FFireDelay := 40;
        end;
     stUs:
        begin
           inherited CreateDIBSprite( frmMain.dibUs );
           FFireDelay := 60;
        end;
  end;

  ptPosition.Y := Random( frmMain.board.PhysicalHeight );

  if ptPosition.Y < 60 then
     ptPosition.Y := 60;
  if ptPosition.Y < frmMain.board.PhysicalHeight - 60 then
     ptPosition.Y := frmMain.board.PhysicalHeight - 60;

  if Random( 100 ) > 50 then
     begin
        ptPosition.X := 0;
        Destination := Point( frmMain.board.PhysicalWidth, ptPosition.Y );
     end
  else
     begin
        ptPosition.X := frmMain.board.PhysicalWidth;
        Destination := Point( 0, ptPosition.Y );
     end;

  Speed := 8;
end;

procedure TSaucer.Move;
var
  n, nOff: integer;
  s: TShip;
  ptDest: TPoint;

  procedure CheckDistance( s_: TShip );
  var
     n_: integer;
  begin
     if not s_.bDead then
        begin
           n_ := FudgedDistance( s_ );
           if n_ < n then
              begin
                 n := n_;
                 s := s_;
              end;
        end;
  end;

begin
  inherited Move;
  if not bKill then
     if Abs( ptPosition.X - Destination.X ) < 10 then
        begin
           UnregisterSprite;
           Exit;
        end;
  Inc( FFire );

  if FFire = FFireDelay then
     begin
        FFire := 0;
        if not frmMain.ship1.bDead or not frmMain.ship2.bDead then
           begin
              n := 1000;
              CheckDistance( frmMain.ship1 );
              CheckDistance( frmMain.ship2 );
              with TMissile.CreateMissile( 45 * ( Ord( st ) + 1 ), Random( 255 ) ) do
                 begin
                    Position := self.ptPosition;
                    ptDest := s.Position;
                    nOff := ( 3 - Ord( st ) ) * 10;
                    Inc( ptDest.X, Random( nOff ) - nOff div 2 );
                    Inc( ptDest.Y, Random( nOff ) - nOff div 2 );
                    Destination := ptDest;
                    Tag := 99;
                    RegisterSprite( w );
                 end;
           end;
     end;
end;

function TSaucer.GetScore: integer;
begin
  case st of
     stAthena:
        Result := 200;
     stUs:
        Result := 500;
     stAnders:
        Result := 1000;
  end;
end;

end.
